# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import distutils
import os


def env_to_bool(env_name, default=False):
    env = os.environ.get(env_name)
    if env is None:
        return default
    return bool(distutils.util.strtobool(env))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import unittest

import mock

from timelineio.utils import env_to_bool


@mock.patch('timelineio.utils.os')
class EnvToBoolTestCase(unittest.TestCase):
    def test_positive_values(self, os_patched):
        os_patched.environ.get.return_value = 'True'
        self.assertTrue(env_to_bool('WHATEVER (IT IS MOCKED)'))

        os_patched.environ.get.return_value = '1'
        self.assertTrue(env_to_bool('WHATEVER (IT IS MOCKED)'))

        os_patched.environ.get.return_value = 't'
        self.assertTrue(env_to_bool('WHATEVER (IT IS MOCKED)'))

        os_patched.environ.get.return_value = 'y'
        self.assertTrue(env_to_bool('WHATEVER (IT IS MOCKED)'))

        os_patched.environ.get.return_value = 'yes'
        self.assertTrue(env_to_bool('WHATEVER (IT IS MOCKED)'))

    def test_negative_values(self, os_patched):
        os_patched.environ.get.return_value = 'False'
        self.assertFalse(env_to_bool('WHATEVER (IT IS MOCKED)'))

        os_patched.environ.get.return_value = '0'
        self.assertFalse(env_to_bool('WHATEVER (IT IS MOCKED)'))

        os_patched.environ.get.return_value = 'f'
        self.assertFalse(env_to_bool('WHATEVER (IT IS MOCKED)'))

        os_patched.environ.get.return_value = 'n'
        self.assertFalse(env_to_bool('WHATEVER (IT IS MOCKED)'))

        os_patched.environ.get.return_value = 'no'
        self.assertFalse(env_to_bool('WHATEVER (IT IS MOCKED)'))

    def test_invalid_values(self, os_patched):
        os_patched.environ.get.return_value = 'Hamster'
        self.assertRaises(ValueError, env_to_bool, 'WHATEVER (IT IS MOCKED)')

    def test_defaults(self, os_patched):
        os_patched.environ.get.return_value = None
        self.assertFalse(env_to_bool('WHATEVER (IT IS MOCKED)'))
        self.assertFalse(env_to_bool('WHATEVER (IT IS MOCKED)', default=False))
        self.assertTrue(env_to_bool('WHATEVER (IT IS MOCKED)', default=True))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from timelineio.resources import TimelineIOEventResource, TimelineIOCustomerResource, TimelineIOAuthResource


class TimelineIOClient(object):
    def __init__(self, base_api_url, auth_credentials=None, auth_token=None):
        self.base_api_url = base_api_url
        self.auth_credentials = auth_credentials
        self.auth_token = auth_token

    @property
    def base_resource_parameters(self):
        return {
            'base_api_url': self.base_api_url,
            'auth_credentials': self.auth_credentials,
            'auth_token': self.auth_token,
        }

    @property
    def Event(self):
        return TimelineIOEventResource(**self.base_resource_parameters)

    @property
    def Customer(self):
        return TimelineIOCustomerResource(**self.base_resource_parameters)

    @property
    def Auth(self):
        return TimelineIOAuthResource(**self.base_resource_parameters)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
import sys

import requests

from timelineio import __version__
from timelineio.exceptions import TimelineIOAPIError


class TimelineIORequestMixin(object):
    def __init__(self, base_api_url, auth_credentials, auth_token):
        self.base_api_url = base_api_url
        self.auth_credentials = auth_credentials
        self.auth_token = auth_token

    def get_full_endpoint(self, path):
        return '{}{}'.format(self.base_api_url, path)

    def _request(self, requests_method, url, data=None, params=None):
        auth = None
        headers = {
            'user-agent': 'timelineio/{} python/{}'.format(__version__, sys.version.split()[0]),
        }

        if self.auth_token:
            headers['Authorization'] = 'Token {}'.format(self.auth_token)
        elif self.auth_credentials:
            auth = self.auth_credentials

        try:
            return requests_method(url, auth=auth, headers=headers, data=data, params=params)
        except requests.exceptions.RequestException as e:
            raise TimelineIOAPIError('{}'.format(e))


class TimelineIOBaseResource(TimelineIORequestMixin):
    endpoint = ''  # Must be set on children classes.

    def get_full_list_endpoint(self):
        return self.get_full_endpoint(self.endpoint)

    def get_full_details_endpoint(self, identifier):
        return '{}/{}'.format(self.get_full_list_endpoint(), identifier)

    def list(self, **params):
        '''
        Fetches a list of resources on API. Use params for endpoint query params (pagination, filtering, ordering, etc)
        Example of params:
        - limit, offset (for pagination - defaults to 30 and 0)
        - object_type, object_id, human_identifier (for filtering)
        - fields, exclude (for reducing API response payload)
        - ordering (for... guess what?)
        '''
        params.setdefault('limit', 30)
        params.setdefault('offset', 0)
        return self._request(requests.get, self.get_full_list_endpoint(), params=params)

    def create(self, **data):
        return self._request(requests.post, self.get_full_list_endpoint(), data=data)

    def retrieve(self, identifier):
        return self._request(requests.get, self.get_full_details_endpoint(identifier))

    def update(self, identifier, **data):
        return self._request(requests.patch, self.get_full_details_endpoint(identifier), data=data)

    def delete(self, identifier):
        return self._request(requests.delete, self.get_full_details_endpoint(identifier))


class TimelineIOEventResource(TimelineIOBaseResource):
    endpoint = '/api/events'


class TimelineIOCustomerResource(TimelineIOBaseResource):
    endpoint = '/api/customers'


class TimelineIOAuthResource(TimelineIORequestMixin):
    def login(self, email, password):
        data = {
            'email': email,
            'password': password,
        }
        return self._request(requests.post, self.get_full_endpoint('/api/auth/login'), data=data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from datetime import datetime
import os

from timelineio.clients import TimelineIOClient
from timelineio.utils import env_to_bool


def create_event(base_api_url, auth_token, message, timestamp, object_id, object_type, human_identifier):
    client = TimelineIOClient(base_api_url=base_api_url, auth_token=auth_token)
    return client.Event.create(
        message=message,
        timestamp=timestamp,
        object_id=object_id,
        object_type=object_type,
        human_identifier=human_identifier
    )

if env_to_bool('TIMELINEIO_ASYNC'):
    from celery import shared_task
    create_event = shared_task(create_event)
    create_event = create_event.delay


class TimelineIOModelMixin(object):
    TIMELINEIO_IDENTIFIER_FIELD = 'id'
    TIMELINEIO_HUMAN_IDENTIFIER_FIELD = None

    def _fully_qualified_object_path(self):
        return '{}.{}'.format(self.__class__.__module__, self.__class__.__name__)

    def timelineio(self, message, timestamp=None):
        if not env_to_bool('TIMELINEIO_ENABLED', default=True):
            return

        if not timestamp:
            timestamp = datetime.utcnow()

        try:
            base_api_url = os.environ['TIMELINEIO_URL']
            auth_token = os.environ['TIMELINEIO_ACCESS_KEY']
        except KeyError:
            raise RuntimeError(
                'In order to use TimelineIO you *must* set both TIMELINEIO_URL and TIMELINEIO_ACCESS_KEY env vars.'
            )

        object_id = str(getattr(self, self.TIMELINEIO_IDENTIFIER_FIELD))
        object_type = self._fully_qualified_object_path()
        timestamp = timestamp.isoformat()

        if self.TIMELINEIO_HUMAN_IDENTIFIER_FIELD:
            human_identifier = getattr(self, self.TIMELINEIO_HUMAN_IDENTIFIER_FIELD)
        else:
            human_identifier = '{}'.format(self)

        return create_event(base_api_url, auth_token, message, timestamp, object_id, object_type, human_identifier)



from setuptools import setup
from timelineio import __version__

BASE_CVS_URL = 'http://github.com/filwaitman/timelineio'

setup(
    name='timelineio',
    packages=['timelineio', ],
    version=__version__,
    author='Filipe Waitman',
    author_email='filwaitman@gmail.com',
    install_requires=[x.strip() for x in open('requirements.txt').readlines()],
    url=BASE_CVS_URL,
    download_url='{}/tarball/{}'.format(BASE_CVS_URL, __version__),
    test_suite='tests',
    tests_require=[x.strip() for x in open('requirements_test.txt').readlines()],
    keywords=[],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ],
)
